@@ -56,22 +56,24 @@ struct _GnomeDbBasicFormClass
 /* 
  * Generic widget's methods 
 */
-guint      gnome_db_basic_form_get_type                 (void);
-GtkWidget *gnome_db_basic_form_new                      (GnomeDbDict *dict, GnomeDbDataSet *data_set);
-GtkWidget *gnome_db_basic_form_new_in_layout            (GnomeDbDict *dict, GnomeDbDataSet *data_set, 
-							 GtkWidget *layout, GHashTable *box_widgets);
-GtkWidget *gnome_db_basic_form_new_in_dialog            (GnomeDbDict *dict, GnomeDbDataSet *data_set, GtkWindow *parent,
-							 const gchar *title, const gchar *header);
-gboolean   gnome_db_basic_form_is_valid                 (GnomeDbBasicForm *form);
-gboolean   gnome_db_basic_form_has_been_changed         (GnomeDbBasicForm *form);
-void       gnome_db_basic_form_reset                    (GnomeDbBasicForm *form);
-void       gnome_db_basic_form_set_current_as_orig      (GnomeDbBasicForm *form);
+guint           gnome_db_basic_form_get_type                 (void);
+GtkWidget      *gnome_db_basic_form_new                      (GnomeDbDict *dict, GnomeDbDataSet *data_set);
+GtkWidget      *gnome_db_basic_form_new_in_layout            (GnomeDbDict *dict, GnomeDbDataSet *data_set, 
+							      GtkWidget *layout, GHashTable *box_widgets);
+GtkWidget      *gnome_db_basic_form_new_in_dialog            (GnomeDbDict *dict, GnomeDbDataSet *data_set, GtkWindow *parent,
+							      const gchar *title, const gchar *header);
+GnomeDbDataSet *gnome_db_basic_form_get_data_set             (GnomeDbBasicForm *form);
+gboolean        gnome_db_basic_form_is_valid                 (GnomeDbBasicForm *form);
+gboolean        gnome_db_basic_form_has_been_changed         (GnomeDbBasicForm *form);
+void            gnome_db_basic_form_reset                    (GnomeDbBasicForm *form);
+void            gnome_db_basic_form_set_current_as_orig      (GnomeDbBasicForm *form);
 
-void       gnome_db_basic_form_show_entries_actions     (GnomeDbBasicForm *form, gboolean show_actions);
-void       gnome_db_basic_form_entry_show               (GnomeDbBasicForm *form, GnomeDbParameter *param, gboolean show);
-void       gnome_db_basic_form_entry_set_sensitive      (GnomeDbBasicForm *form, GnomeDbParameter *param, gboolean sensitive);
-void       gnome_db_basic_form_set_entries_auto_default (GnomeDbBasicForm *form, gboolean auto_default);
-void       gnome_db_basic_form_set_entries_default      (GnomeDbBasicForm *form);
+void            gnome_db_basic_form_show_entries_actions     (GnomeDbBasicForm *form, gboolean show_actions);
+void            gnome_db_basic_form_entry_show               (GnomeDbBasicForm *form, GnomeDbParameter *param, gboolean show);
+void            gnome_db_basic_form_entry_set_sensitive      (GnomeDbBasicForm *form, GnomeDbParameter *param, 
+							      gboolean sensitive);
+void            gnome_db_basic_form_set_entries_auto_default (GnomeDbBasicForm *form, gboolean auto_default);
+void            gnome_db_basic_form_set_entries_default      (GnomeDbBasicForm *form);
 
 G_END_DECLS
 
