@@ -1,8 +1,9 @@
 /* GNOME DB library
- * Copyright (C) 1999-2002 The GNOME Foundation
+ * Copyright (C) 1999 - 2005 The GNOME Foundation
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -21,35 +22,53 @@
  */
 
 #include <string.h>
-#include <libgnomedb/gnome-db-config.h>
+#include <libgda/gda-client.h>
+#include <libgda/gda-config.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-druid-page-edge.h>
 #include <libgnomeui/gnome-druid-page-standard.h>
 #include <libgnomedb/gnome-db-dsn-config-druid.h>
 #include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
-#include "gnome-db-i18n.h"
+#include <libgnomedb/gnome-db-data-set.h>
+#include <libgnomedb/gnome-db-parameter.h>
+#include <libgnomedb/gnome-db-basic-form.h>
+#include <glib/gi18n-lib.h>
 
 #define PARENT_TYPE GNOME_TYPE_DRUID
 
 struct _GnomeDbDsnConfigDruidPrivate {
+	GdaClient *client;
 	GdaDataSourceInfo *dsn_info;
 
 	GdkPixbuf *logo;
 
 	/* widgets */
 	GtkWidget *start_page;
+
 	GtkWidget *general_page;
 	GtkWidget *general_name;
 	GtkWidget *general_provider;
 	GtkWidget *general_description;
 	GtkWidget *general_username;
 	GtkWidget *general_password;
+	GtkWidget *general_is_global;
+	gchar     *create_db_spec;
+
+	GtkWidget *choose_page;
+	GtkWidget *choose_toggle;
+
+	GtkWidget *newdb_page;
+	GtkWidget *newdb_box;
+	GtkWidget *newdb_params;
+
 	GtkWidget *provider_page;
 	GtkWidget *provider_container;
 	GtkWidget *provider_detail;
+
 	GHashTable *provider_params;
 	GHashTable *remembered_values;
+
 	GtkWidget *end_page;
 };
 
@@ -63,7 +82,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static gint config_druid_signals[LAST_SIGNAL] = { 0, };
+static guint config_druid_signals[LAST_SIGNAL] = { 0, };
 static GObjectClass *parent_class = NULL;
 
 /*
@@ -131,48 +150,186 @@ remove_remembered_hash (gpointer key, gpointer value, gpointer user_data)
  * Callbacks
  */
 
+static gchar *
+get_specs_database_creation (GnomeDbDsnConfigDruid *druid)
+{
+	if (! druid->priv->create_db_spec) {
+		if (!druid->priv->client)
+			druid->priv->client = gda_client_new ();
+		
+		druid->priv->create_db_spec = gda_client_get_specs_to_create_database (druid->priv->client, 
+								gnome_db_provider_selector_get_selected_provider (
+								GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+	}
+
+	return druid->priv->create_db_spec;
+}
+
 static void
 druid_cancelled_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, gpointer user_data)
 {
 	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
 
 	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid));
-	g_signal_emit (G_OBJECT (druid), config_druid_signals[FINISHED], 0, TRUE);
+	g_signal_emit_by_name (G_OBJECT (druid), "finished", TRUE);
 }
 
 static void
-druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, gpointer user_data)
+druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, GnomeDbDsnConfigDruid *druid)
 {
-	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
+	gboolean allok = TRUE;
+	GString *cnc_string = NULL;
 
 	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid));
 
 	/* set the internal dsn_info with the values of the widgets */
-	if (druid->priv->dsn_info)
-		gnome_db_config_free_data_source_info (druid->priv->dsn_info);
+	if (druid->priv->dsn_info) {
+		gda_data_source_info_free (druid->priv->dsn_info);
+		druid->priv->dsn_info = NULL;
+	}
 
-	druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
-	druid->priv->dsn_info->name =
-		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_name)));
-	druid->priv->dsn_info->provider =
-		g_strdup (gnome_db_option_menu_get_selection (
-				  GTK_OPTION_MENU (druid->priv->general_provider)));
-	druid->priv->dsn_info->cnc_string = params_to_string (druid);
-	druid->priv->dsn_info->description =
-		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_description)));
-	druid->priv->dsn_info->username =
-		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_username)));
-	druid->priv->dsn_info->password =
-		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_password)));
+	/* New database creation first */
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle))) {
+		/* make a list of GdaParameter parameters */
+		GnomeDbDataSet *dset;
+
+		dset = gnome_db_basic_form_get_data_set (GNOME_DB_BASIC_FORM (druid->priv->newdb_params));
+		if (!dset) 
+			return;
+
+		if (!gnome_db_data_set_is_valid (dset)) {
+			gnome_db_show_error (_("Missing mandatory informmation, to create database"));
+			gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
+			return;
+		}
+		else {
+			GdaParameterList *plist = NULL;
+			GSList *params;
+			GdaParameter *gda_param;
+			GnomeDbParameter *param;
+			gchar *str;
+			GError *error = NULL;
+			GList *dsn_params;
+
+			GdaProviderInfo *prov_info;
+
+			params = dset->parameters;
+			while (params) {
+				param = GNOME_DB_PARAMETER (params->data);
+				gda_param = gda_parameter_new_from_value ("", gnome_db_parameter_get_value (param));
+				g_object_get (G_OBJECT (param), "string_id", &str, NULL);
+				if (str) {
+					gda_parameter_set_name (gda_param, str);
+					g_free (str);
+				}
+				else
+					gda_parameter_set_name (gda_param, gnome_db_base_get_name (GNOME_DB_BASE (param)));	
+				if (!plist)
+					plist = gda_parameter_list_new ();
+				gda_parameter_list_add_parameter (plist, gda_param);
+
+				params = g_slist_next (params);
+			}
+
+			allok = gda_client_create_database (druid->priv->client, 
+							    gnome_db_provider_selector_get_selected_provider (
+								    GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)),
+							    plist, &error);
+			if (!allok) {
+				str = g_strdup_printf (_("Error creating database: %s"), 
+						       error && error->message ? error->message : _("Unknown error"));
+				gnome_db_show_error (str);
+				g_free (str);
+
+				gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
+				gda_parameter_list_free (plist);
+				return;
+			}
+
+			/* make the connection string for the data source */
+			prov_info = gda_config_get_provider_by_name (gnome_db_provider_selector_get_selected_provider (
+									     GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+			g_return_if_fail (prov_info);
+			dsn_params = prov_info->gda_params;
+			while (dsn_params) {
+				GdaParameter *param;
+
+				param = gda_parameter_list_find (plist, ((GdaProviderParameterInfo*)(dsn_params->data))->name);
+				if (param) {
+					const GdaValue *value;
+
+					value = gda_parameter_get_value (param);
+					if (value && !gda_value_is_null (value)) {
+						if (dsn_params == prov_info->gda_params)
+							cnc_string = g_string_new ("");
+						else
+							g_string_append (cnc_string, "; ");
+						str = gda_value_stringify (value);
+						g_string_append_printf (cnc_string, "%s=%s", 
+									((GdaProviderParameterInfo*)(dsn_params->data))->name, str);
+						g_free (str);
+					}
+				}
+
+				dsn_params = g_list_next (dsn_params);
+			}
+
+			gda_parameter_list_free (plist);
+		}
+	}
+
+	/* Data source declaration */
+	if (allok) {
+		druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
+		druid->priv->dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_name)));
+		druid->priv->dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider (
+								    GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+		if (cnc_string) {
+			g_printf ("CNC_STRING:%s\n", cnc_string->str);
+			druid->priv->dsn_info->cnc_string = cnc_string->str;
+			g_string_free (cnc_string, FALSE);
+		}
+		else
+			druid->priv->dsn_info->cnc_string = params_to_string (druid);
+		druid->priv->dsn_info->description =
+			g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_description)));
+		druid->priv->dsn_info->username =
+			g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_username)));
+		druid->priv->dsn_info->password =
+			g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_password)));
+		if (gda_config_can_modify_global_config ())
+			druid->priv->dsn_info->is_global = gtk_toggle_button_get_active 
+				(GTK_TOGGLE_BUTTON (druid->priv->general_is_global));
+		else
+			druid->priv->dsn_info->is_global = FALSE;
+
+		allok = gda_config_save_data_source_info (druid->priv->dsn_info);
+	}
 
 	/* notify listeners */
-	g_signal_emit (G_OBJECT (druid), config_druid_signals[FINISHED], 0, FALSE);
+	g_signal_emit (G_OBJECT (druid), config_druid_signals[FINISHED], 0, !allok);
+}
+
+static void
+provider_changed_cb (GtkWidget *combo, GnomeDbDsnConfigDruid *druid)
+{
+	/* clean any previous Provider specific stufv */
+	if (druid->priv->newdb_params) {
+		gtk_widget_destroy (druid->priv->newdb_params);
+		druid->priv->newdb_params = NULL;
+	}
+
+	/* is the database creation supported by the chosen provider? */
+	if (druid->priv->create_db_spec) {
+		g_free (druid->priv->create_db_spec);
+		druid->priv->create_db_spec = NULL;
+	}
 }
 
 static gboolean
 general_next_pressed_cb (GnomeDruidPage *druid_page,
 			 GtkWidget *druid_widget,
-			 gpointer user_data)
+			 GnomeDbDsnConfigDruid *druid)
 {
 	GdaDataSourceInfo *dsn_info;
 	GdaProviderInfo *prov_info;
@@ -180,7 +337,7 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 	gint i;
 	const gchar *name;
 	const gchar *provider;
-	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
+	gchar *str;
 
 	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), TRUE);
 
@@ -200,9 +357,9 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 		}
 	}
 
-	dsn_info = gnome_db_config_find_data_source (name);
+	dsn_info = gda_config_find_data_source (name);
 	if (dsn_info != NULL) {
-		gnome_db_config_free_data_source_info (dsn_info);
+		gda_data_source_info_free (dsn_info);
 		gnome_db_show_error (_("An existing data source is already registered under the same name"));
 		gtk_widget_grab_focus (druid->priv->general_name);
 		return TRUE;
@@ -220,7 +377,8 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 		}
 	}
 
-	provider = gnome_db_option_menu_get_selection (GTK_OPTION_MENU (druid->priv->general_provider));
+	provider =  gnome_db_provider_selector_get_selected_provider (
+		GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider));
 	if (!provider) {
 		gnome_db_show_error (_("You must select a provider for the new data source"));
 		gtk_widget_grab_focus (druid->priv->general_provider);
@@ -230,7 +388,10 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 	/* set up widgets for this specific provider */
 	prov_info = gda_config_get_provider_by_name (provider);
 	if (!prov_info) {
-		gnome_db_show_error (_("Provider not found in your configuration"));
+		str = g_strdup_printf (_("Could not find provider %s in the current setup"),
+				       provider);
+		gnome_db_show_error (str);
+		g_free (str);
 		gtk_widget_grab_focus (druid->priv->general_provider);
 		return TRUE;
 	}
@@ -248,7 +409,7 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 		GtkWidget *label;
 		GtkWidget *widget;
 
-		param_info = (const gchar *) l->data;
+		param_info = (GdaProviderParameterInfo *) l->data;
 		if (param_info) {
 			GHashTable *values;
 			gchar *title;
@@ -280,24 +441,90 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 	gtk_box_pack_start (GTK_BOX (druid->priv->provider_container),
 			    druid->priv->provider_detail, TRUE, TRUE, 0);
 
-	gda_provider_info_free (prov_info);
+	str = get_specs_database_creation (druid);
+	if (str) 
+		return FALSE;
+	else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle), FALSE);
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->provider_page));
+		return TRUE;
+	}	
+}
 
-	return FALSE;
+static gboolean
+choose_next_pressed_cb (GnomeDruidPage *druid_page,
+			GtkWidget *druid_widget,
+			GnomeDbDsnConfigDruid *druid)
+{
+	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), FALSE);
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle))) {
+		/* prepare a new data set for database creation parameters */
+		gchar *str;
+
+		if (! druid->priv->newdb_params) {
+			if (!druid->priv->client)
+				druid->priv->client = gda_client_new ();
+			
+			str = gda_client_get_specs_to_create_database (druid->priv->client, 
+								       gnome_db_provider_selector_get_selected_provider (
+									       GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider)));
+			if (str) {
+				GnomeDbDataSet *dset;
+				GError *error = NULL;
+				
+				dset = GNOME_DB_DATA_SET (gnome_db_data_set_new_from_spec (NULL, str, &error));
+				if (dset) {
+					druid->priv->newdb_params = gnome_db_basic_form_new (NULL, dset);
+					gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (druid->priv->newdb_params),
+										  FALSE);
+					g_object_unref (dset);
+				}
+				else {
+					gchar *str2;
+					str2 = g_strdup_printf ("Provider internal error: %s", 
+								error && error->message ? error->message : "???" );
+					druid->priv->newdb_params = gtk_label_new (str2);
+					g_free (str2);
+				}
+				
+				gtk_widget_show (druid->priv->newdb_params);
+				gtk_box_pack_start (GTK_BOX (druid->priv->newdb_box), druid->priv->newdb_params,
+						    TRUE, TRUE, 0);
+				g_free (str);
+			}
+		}
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
+	}
+	else
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->provider_page));
+
+	return TRUE;
+}
+
+static gboolean
+newdb_next_pressed_cb (GnomeDruidPage *druid_page,
+		       GtkWidget *druid_widget,
+		       GnomeDbDsnConfigDruid *druid)
+{
+	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), FALSE);
+
+	gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->end_page));
+
+	return TRUE;
 }
 
 static void
-add_remembered_hash (gpointer key, gpointer value, gpointer user_data)
+add_remembered_hash (gpointer key, gpointer value, GnomeDbDsnConfigDruid *druid)
 {
 	GHashTable *htable;
 	const gchar *provider;
 	const gchar *param_name = key;
 	const gchar *entry_value;
 	GtkWidget *entry = (GtkWidget *) value;
-	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
 
-	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid));
-
-	provider = gnome_db_option_menu_get_selection (GTK_OPTION_MENU (druid->priv->general_provider));
+	provider = gnome_db_provider_selector_get_selected_provider (
+		GNOME_DB_PROVIDER_SELECTOR (druid->priv->general_provider));
 	entry_value = gtk_entry_get_text (GTK_ENTRY (entry));
 
 	htable = g_hash_table_lookup (druid->priv->remembered_values, provider);
@@ -320,14 +547,40 @@ add_remembered_hash (gpointer key, gpointer value, gpointer user_data)
 static gboolean
 provider_back_pressed_cb (GnomeDruidPage *druid_page,
 			  GtkWidget *druid_widget,
-			  gpointer user_data)
+			  GnomeDbDsnConfigDruid *druid)
 {
-	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
-
+	gchar *str;
 	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), TRUE);
 
 	g_hash_table_foreach (druid->priv->provider_params, (GHFunc) add_remembered_hash, druid);
-	return FALSE;
+
+	/*
+	 * is the database creation supported by the chosen provider?
+	 */
+	str = get_specs_database_creation (druid);
+	if (str) {
+		/* database creation is supported */
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->choose_page));
+	}
+	else 
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->general_page));
+
+	return TRUE;
+}
+
+static gboolean
+end_back_pressed_cb (GnomeDruidPage *druid_page,
+			  GtkWidget *druid_widget,
+			  GnomeDbDsnConfigDruid *druid)
+{
+	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), TRUE);
+	
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (druid->priv->choose_toggle)))
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->newdb_page));
+	else
+		gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (druid->priv->provider_page));
+
+	return TRUE;
 }
 
 /*
@@ -350,6 +603,7 @@ gnome_db_dsn_config_druid_class_init (GnomeDbDsnConfigDruidClass *klass)
 			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE, 1, G_TYPE_INT);
 
+	klass->finished = NULL;
 	object_class->finalize = gnome_db_dsn_config_druid_finalize;
 }
 
@@ -367,6 +621,7 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	druid->priv->provider_detail = NULL;
 	druid->priv->provider_params = g_hash_table_new (g_str_hash, g_str_equal);
 	druid->priv->remembered_values = g_hash_table_new (g_str_hash, g_str_equal);
+	druid->priv->create_db_spec = NULL;
 
 	/* load icons */
 	druid->priv->logo = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
@@ -374,7 +629,9 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	/* set up widgets */
 	gnome_druid_set_show_help (GNOME_DRUID (druid), FALSE);
 
-	/* start page */
+	/* 
+	 * start page
+	 */
 	druid->priv->start_page = gnome_druid_page_edge_new_with_vals (
 		GNOME_EDGE_START, TRUE, _("Add a new data source..."),
 		_("This druid will guide you through the process of\n"
@@ -387,17 +644,20 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 
 	g_signal_connect (G_OBJECT (druid->priv->start_page), "cancel", G_CALLBACK (druid_cancelled_cb), druid);
 
-	/* general info page */
+	/* 
+	 * general info page 
+	 */
 	druid->priv->general_page = gnome_druid_page_standard_new_with_vals (
 		_("General Information"), druid->priv->logo, NULL);
 
 	label = gnome_db_new_label_widget (NULL);
 	gtk_label_set_markup (GTK_LABEL (label),
-		_("The following fields represent the basic information\n"
-		  "items for your new data source. Some of them (description,\n"
-		  "username) are optional, whereas the others are mandatory.\n"
-		  "If you don't know which provider to use, just select\n"
-		  "the default one."));
+			      _("The following fields represent the basic information "
+				"items for your new data source. Some of them (description, "
+				"username) are optional, whereas the others are mandatory. "
+				"If you don't know which provider to use, just select "
+				"the default one."));
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gnome_druid_page_standard_append_item (
 		GNOME_DRUID_PAGE_STANDARD (druid->priv->general_page), NULL, label, NULL);
 
@@ -406,11 +666,22 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 		GNOME_DRUID_PAGE_STANDARD (druid->priv->general_page),
 		_("Data source _name:"), druid->priv->general_name, NULL);
 
+	if (gda_config_can_modify_global_config ()) {
+		druid->priv->general_is_global = gtk_check_button_new ();
+		gnome_druid_page_standard_append_item (
+			GNOME_DRUID_PAGE_STANDARD (druid->priv->general_page),
+			_("System wide data source:"), druid->priv->general_is_global, NULL);
+	}
+	else
+		druid->priv->general_is_global = NULL;
+
 	druid->priv->general_provider = gnome_db_provider_selector_new ();
 	gtk_widget_show (druid->priv->general_provider);
 	gnome_druid_page_standard_append_item (
 		GNOME_DRUID_PAGE_STANDARD (druid->priv->general_page),
 		_("_Provider:"), druid->priv->general_provider, NULL);
+	g_signal_connect (G_OBJECT (druid->priv->general_provider), "changed",
+			  G_CALLBACK (provider_changed_cb), druid);
 
 	druid->priv->general_description = gnome_db_new_entry_widget (0, TRUE);
 	gnome_druid_page_standard_append_item (
@@ -437,16 +708,79 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	g_signal_connect (G_OBJECT (druid->priv->general_page), "next",
 			  G_CALLBACK (general_next_pressed_cb), druid);
 
-	/* provider parameters page */
+	/*
+	 * Choose between existing database or create a new one
+	 */
+	druid->priv->choose_page = gnome_druid_page_standard_new_with_vals (
+		_("Use existing database"), druid->priv->logo, NULL);
+	label = gnome_db_new_label_widget (NULL);
+	gtk_label_set_markup (GTK_LABEL (label),
+			      _("This page lets you choose between using an existing database "
+				"or to create a new database to use with this new data source"));
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+
+	gnome_druid_page_standard_append_item (
+		GNOME_DRUID_PAGE_STANDARD (druid->priv->choose_page), NULL, label, NULL);
+
+	druid->priv->choose_toggle = gtk_check_button_new_with_label (_("Create a new database"));
+	gnome_druid_page_standard_append_item (
+		GNOME_DRUID_PAGE_STANDARD (druid->priv->choose_page), NULL,
+		druid->priv->choose_toggle, NULL);
+
+	gtk_widget_show (druid->priv->choose_page);
+	gnome_druid_append_page (GNOME_DRUID (druid),
+				 GNOME_DRUID_PAGE (druid->priv->choose_page));
+
+	g_signal_connect (G_OBJECT (druid->priv->choose_page), "next",
+			  G_CALLBACK (choose_next_pressed_cb), druid);
+	g_signal_connect (G_OBJECT (druid->priv->choose_page), "cancel", 
+			  G_CALLBACK (druid_cancelled_cb), druid);
+
+	/*
+	 * New database page
+	 */
+	druid->priv->newdb_page = gnome_druid_page_standard_new_with_vals (
+		_("New database creation"), druid->priv->logo, NULL);
+	label = gnome_db_new_label_widget (NULL);
+	gtk_label_set_markup (GTK_LABEL (label),
+			      _("The following fields represent the information needed by the "
+				"provider you selected in the previous page to create a new database "
+				"(mandatory fields are marked in bold text)."
+				"This information is provider-specific, so check "
+				"the manual for more information."));
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gnome_druid_page_standard_append_item (
+		GNOME_DRUID_PAGE_STANDARD (druid->priv->newdb_page), NULL, label, NULL);
+
+	druid->priv->newdb_box = gtk_vbox_new (FALSE, 0);
+	gnome_druid_page_standard_append_item (
+		GNOME_DRUID_PAGE_STANDARD (druid->priv->newdb_page), NULL,
+		druid->priv->newdb_box, NULL);
+	druid->priv->newdb_params = NULL;
+	
+	g_signal_connect (G_OBJECT (druid->priv->newdb_page), "cancel", 
+			  G_CALLBACK (druid_cancelled_cb), druid);
+	g_signal_connect (G_OBJECT (druid->priv->newdb_page), "next", 
+			  G_CALLBACK (newdb_next_pressed_cb), druid);
+
+	gtk_widget_show (druid->priv->newdb_page);
+	gnome_druid_append_page (GNOME_DRUID (druid),
+				 GNOME_DRUID_PAGE (druid->priv->newdb_page));
+
+	/* 
+	 * provider parameters page 
+	 */
 	druid->priv->provider_page = gnome_druid_page_standard_new_with_vals (
 		_("Provider Parameters"), druid->priv->logo, NULL);
 
 	label = gnome_db_new_label_widget (NULL);
 	gtk_label_set_markup (GTK_LABEL (label),
-		_("The following fields represent the information needed by the\n"
-		  "provider you selected in the previous page to open a given\n"
-		  "connection. This information is provider-specific, so check\n"
-		  "the manual for more information."));
+			      _("The following fields represent the information needed by the "
+				"provider you selected in the previous page to open a given "
+				"connection. "
+				"This information is provider-specific, so check "
+				"the manual for more information."));
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gnome_druid_page_standard_append_item (
 		GNOME_DRUID_PAGE_STANDARD (druid->priv->provider_page), NULL, label, NULL);
 
@@ -464,7 +798,9 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	g_signal_connect (G_OBJECT (druid->priv->provider_page), "back",
 			  G_CALLBACK (provider_back_pressed_cb), druid);
 
-	/* end page */
+	/* 
+	 * end page 
+	 */
 	druid->priv->end_page = gnome_druid_page_edge_new_with_vals (
 		GNOME_EDGE_FINISH, TRUE, _("All information retrieved"),
 		_("All information needed to create a new data source\n"
@@ -476,6 +812,7 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	gnome_druid_append_page (GNOME_DRUID (druid),
 				 GNOME_DRUID_PAGE (druid->priv->end_page));
 
+	g_signal_connect (G_OBJECT (druid->priv->end_page), "back", G_CALLBACK (end_back_pressed_cb), druid);
 	g_signal_connect (G_OBJECT (druid->priv->end_page), "cancel", G_CALLBACK (druid_cancelled_cb), druid);
 	g_signal_connect (G_OBJECT (druid->priv->end_page), "finish", G_CALLBACK (druid_finished_cb), druid);
 }
@@ -488,8 +825,12 @@ gnome_db_dsn_config_druid_finalize (GObject *object)
 	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid));
 
 	/* free memory */
+	if (druid->priv->client)
+		g_object_unref (druid->priv->client);
+
 	gdk_pixbuf_unref (druid->priv->logo);
-	gnome_db_config_free_data_source_info (druid->priv->dsn_info);
+	if (druid->priv->dsn_info)
+		gda_data_source_info_free (druid->priv->dsn_info);
 
 	g_hash_table_foreach_remove (druid->priv->provider_params,
 				     (GHRFunc) remove_hash_func,
@@ -503,6 +844,9 @@ gnome_db_dsn_config_druid_finalize (GObject *object)
 	g_hash_table_destroy (druid->priv->remembered_values);
 	druid->priv->remembered_values = NULL;
 
+	if (druid->priv->create_db_spec)
+		g_free (druid->priv->create_db_spec);
+
 	g_free (druid->priv);
 	druid->priv = NULL;
 
