@@ -234,6 +234,7 @@ GtkWidget *
 gnome_db_form_new_with_gda_model (GnomeDbDict *dict, GdaDataModel *model)
 {
 	TO_IMPLEMENT;
+	return NULL;
 }
 
 
@@ -597,7 +598,7 @@ gnome_db_form_initialize (GnomeDbForm *form, GnomeDbQuery *orig_query, GtkWidget
 	realform = gnome_db_basic_form_new_in_layout (dict, form->priv->data_set, layout, box_widgets);
 	gtk_notebook_append_page (GTK_NOTEBOOK (nb), realform, NULL);
 	gtk_widget_show (realform);
-	if (!layout && gnome_db_data_proxy_is_read_only)
+	if (!layout && gnome_db_data_proxy_is_read_only (form->priv->proxy))
 		gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (realform), FALSE);
 	form->priv->basic_form = realform;
 
@@ -785,6 +786,7 @@ modif_buttons_make (GnomeDbForm *form)
 	gtk_ui_manager_add_ui_from_string (ui, ui_actions_info, -1, NULL);
 	form->priv->uimanager = ui;
 	form->priv->modif_all = gtk_ui_manager_get_widget (ui, "/ToolBar");
+	g_object_set (G_OBJECT (form->priv->modif_all), "toolbar-style", GTK_TOOLBAR_ICONS, NULL);
 	gtk_widget_show (form->priv->modif_all);
 	gtk_box_pack_start (GTK_BOX (hbox), form->priv->modif_all, TRUE, TRUE, 0);
 
